@@ -10,11 +10,11 @@ ec2query_build <- function(request) {
   if (!is_presigned(request)) {
     request$http_request$method <- "POST"
     request$http_request$header["Content-Type"] <- "application/x-www-form-urlencoded; charset=utf-8"
-    request$body <- encode(body)
+    request$body <- build_query_string(body)
     request$http_request$body <- request$body
   } else {
     request$http_request$method <- "GET"
-    request$http_request$url$raw_query <- encode(body)
+    request$http_request$url$raw_query <- build_query_string(body)
   }
   return(request)
 }
